<?php

namespace Intergo\VerifyClient;

use Intergo\BaseClient\Responses\BaseResponseInterface;

abstract class VerifierServiceManager
{
    /** @var VerifierClient */
    protected $client;


    /** @var string */
    protected $token;

    public function __construct(string $baseUrl, string $traceId, string $token = null)
    {
        $this->client = new VerifierClient($baseUrl, $traceId);
        $this->token = $token;
    }

    /**
     * @param string $action
     * @param BaseResponseInterface $result
     * @return BaseResponseInterface
     */
    abstract public function response(string $action, BaseResponseInterface $result): BaseResponseInterface;

    /**
     * @param $endpoint
     * @param  array  $urlParams
     * @param  array  $bodyParams
     * @return BaseResponseInterface
     */
    protected function executeAuth($endpoint, array $urlParams = [], array $bodyParams = []): BaseResponseInterface
    {
        $result = $this->client->executeAuth($endpoint, $this->token, $urlParams, $bodyParams);
        return $this->response($endpoint, $result);
    }

    /***********************Applications****************************/
    public function getApplication($id)
    {
        return $this->executeAuth(__FUNCTION__, compact('id'));
    }
    public function getApplications()
    {
        return $this->executeAuth(__FUNCTION__);
    }
    public function storeApplication($name, $user_id)
    {
        return $this->executeAuth(__FUNCTION__, [],compact('name', 'user_id'));
    }
    public function updateApplication($id, $name, $user_id)
    {
        return $this->executeAuth(__FUNCTION__, compact('id'),compact('name', 'user_id'));
    }
    public function deleteApplication($id)
    {
        return $this->executeAuth(__FUNCTION__, compact('id'));
    }
    /***********************Applications Settings****************************/
    public function getApplicationSetting($id)
    {
        return $this->executeAuth(__FUNCTION__, compact('id'));
    }
    public function storeApplicationSetting($channel, $min_length, $max_length, $max_tries, $ttl, $password_type, $callback_url, $application_id)
    {
        return $this->executeAuth(__FUNCTION__, [], compact(
            'channel',
            'min_length',
            'max_length',
            'max_tries',
            'ttl',
            'password_type',
            'callback_url',
            'application_id'));
    }
    public function updateApplicationSetting($id, $channel, $min_length, $max_length, $max_tries, $ttl, $password_type, $callback_url, $application_id)
    {
        return $this->executeAuth(__FUNCTION__, compact('id'),compact(
            'channel',
            'min_length',
            'max_length',
            'max_tries',
            'ttl',
            'password_type',
            'callback_url',
            'application_id'));
    }
    public function deleteApplicationSetting($id)
    {
        return $this->executeAuth(__FUNCTION__, compact('id'));
    }
    /***********************Billing Settings****************************/
    public function getBillingSetting($id)
    {
        return $this->executeAuth(__FUNCTION__, compact('id'));
    }
    public function storeBillingSetting($channel, $our_cost, $cost_success, $cost_fail, $application_id)
    {
        return $this->executeAuth(__FUNCTION__, [], compact(
            'channel',
            'our_cost',
            'cost_success',
            'cost_fail',
            'application_id'));
    }
    public function updateBillingSetting($id, $channel, $our_cost, $cost_success, $cost_fail, $application_id)
    {
        return $this->executeAuth(__FUNCTION__, compact('id'),compact(
            'channel',
            'our_cost',
            'cost_success',
            'cost_fail',
            'application_id'));
    }
    public function deleteBillingSetting($id)
    {
        return $this->executeAuth(__FUNCTION__, compact('id'));
    }
    /***********************Templates****************************/
    public function getTemplate($id)
    {
        return $this->executeAuth(__FUNCTION__, compact('id'));
    }
    public function storeTemplate($locale, $channel, $title, $message, $application_id)
    {
        return $this->executeAuth(__FUNCTION__, [],compact('locale',
            'channel',
            'title',
            'message',
            'application_id'));
    }
    public function updateTemplate($id, $locale, $channel, $title, $message, $application_id)
    {
        return $this->executeAuth(__FUNCTION__, compact('id'),compact('locale',
            'channel',
            'title',
            'message',
            'application_id'));
    }
    public function deleteTemplate($id)
    {
        return $this->executeAuth(__FUNCTION__, compact('id'));
    }
    /***********************Templates****************************/
    public function create($application_code, $recipient, $reference, $callback_url, $channel)
    {
        return $this->executeAuth(__FUNCTION__, [], compact(
            'application_code',
            'recipient',
            'reference',
            'callback_url',
            'channel',
        ));
    }
    public function confirm($id, $code)
    {
        return $this->executeAuth(__FUNCTION__, [], compact('id', 'code'));
    }
    public function cancel($id)
    {
        return $this->executeAuth(__FUNCTION__, [], compact('id'));
    }
    public function status($id)
    {
        return $this->executeAuth(__FUNCTION__, compact('id'));
    }
}